<?php

namespace Notifications\Notification\Http\Controllers;

use App\Http\Controllers\ResourceController as BaseController;
use Form;
use Notifications\Notification\Http\Requests\NotificationsRequest;
use Notifications\Notification\Interfaces\NotificationsRepositoryInterface;
use Notifications\Notification\Models\Notifications;

/**
 * Resource controller class for notifications.
 */
class NotificationsResourceController extends BaseController
{

    /**
     * Initialize notifications resource controller.
     *
     * @param type NotificationsRepositoryInterface $notifications
     *
     * @return null
     */
    public function __construct(NotificationsRepositoryInterface $notifications)
    {
        parent::__construct();
        $this->repository = $notifications;
        $this->repository
            ->pushCriteria(\Litepie\Repository\Criteria\RequestCriteria::class)
            ->pushCriteria(\Notifications\Notification\Repositories\Criteria\NotificationsResourceCriteria::class);
    }

    /**
     * Display a list of notifications.
     *
     * @return Response
     */
    public function index(NotificationsRequest $request)
    {
        $view = $this->response->theme->listView();

        if ($this->response->typeIs('json')) {
            $function = camel_case('get-' . $view);
            return $this->repository
                ->setPresenter(\Notifications\Notification\Repositories\Presenter\NotificationsPresenter::class)
                ->$function();
        }

        $notifications = $this->repository->paginate();

        return $this->response->title(trans('notification::notifications.names'))
            ->view('notification::notifications.index', true)
            ->data(compact('notifications'))
            ->output();
    }

    /**
     * Display notifications.
     *
     * @param Request $request
     * @param Model   $notifications
     *
     * @return Response
     */
    public function show(NotificationsRequest $request, Notifications $notifications)
    {

        if ($notifications->exists) {
            $view = 'notification::notifications.show';
        } else {
            $view = 'notification::notifications.new';
        }

        return $this->response->title(trans('app.view') . ' ' . trans('notification::notifications.name'))
            ->data(compact('notifications'))
            ->view($view, true)
            ->output();
    }

    /**
     * Show the form for creating a new notifications.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function create(NotificationsRequest $request)
    {

        $notifications = $this->repository->newInstance([]);
        return $this->response->title(trans('app.new') . ' ' . trans('notification::notifications.name')) 
            ->view('notification::notifications.create', true) 
            ->data(compact('notifications'))
            ->output();
    }

    /**
     * Create new notifications.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function store(NotificationsRequest $request)
    {
        try {
            $attributes              = $request->all();
            $attributes['user_id']   = user_id();
            $attributes['user_type'] = user_type();
            $notifications                 = $this->repository->create($attributes);

            return $this->response->message(trans('messages.success.created', ['Module' => trans('notification::notifications.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('notification/notifications/' . $notifications->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('/notification/notifications'))
                ->redirect();
        }

    }

    /**
     * Show notifications for editing.
     *
     * @param Request $request
     * @param Model   $notifications
     *
     * @return Response
     */
    public function edit(NotificationsRequest $request, Notifications $notifications)
    {
        return $this->response->title(trans('app.edit') . ' ' . trans('notification::notifications.name'))
            ->view('notification::notifications.edit', true)
            ->data(compact('notifications'))
            ->output();
    }

    /**
     * Update the notifications.
     *
     * @param Request $request
     * @param Model   $notifications
     *
     * @return Response
     */
    public function update(NotificationsRequest $request, Notifications $notifications)
    {
        try {
            $attributes = $request->all();

            $notifications->update($attributes);
            return $this->response->message(trans('messages.success.updated', ['Module' => trans('notification::notifications.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('notification/notifications/' . $notifications->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('notification/notifications/' . $notifications->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove the notifications.
     *
     * @param Model   $notifications
     *
     * @return Response
     */
    public function destroy(NotificationsRequest $request, Notifications $notifications)
    {
        try {

            $notifications->delete();
            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('notification::notifications.name')]))
                ->code(202)
                ->status('success')
                ->url(guard_url('notification/notifications/0'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('notification/notifications/' . $notifications->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove multiple notifications.
     *
     * @param Model   $notifications
     *
     * @return Response
     */
    public function delete(NotificationsRequest $request, $type)
    {
        try {
            $ids = hashids_decode($request->input('ids'));

            if ($type == 'purge') {
                $this->repository->purge($ids);
            } else {
                $this->repository->delete($ids);
            }

            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('notification::notifications.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('notification/notifications'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/notification/notifications'))
                ->redirect();
        }

    }

    /**
     * Restore deleted notifications.
     *
     * @param Model   $notifications
     *
     * @return Response
     */
    public function restore(NotificationsRequest $request)
    {
        try {
            $ids = hashids_decode($request->input('ids'));
            $this->repository->restore($ids);

            return $this->response->message(trans('messages.success.restore', ['Module' => trans('notification::notifications.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('/notification/notifications'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/notification/notifications/'))
                ->redirect();
        }

    }

}
